@@ -52,6 +52,13 @@
 #include <QtCore/QDir>
 #include <QStandardPaths>
 #include <QThread>
+#include <QLibraryInfo>
+
+#include <QLoggingCategory>
+Q_DECLARE_LOGGING_CATEGORY(LOG_KCRASH)
+
+// logging category for this framework, default: log stuff >= info
+Q_LOGGING_CATEGORY(LOG_KCRASH, "org.kde.kcrash", QtInfoMsg)
 
 #if HAVE_X11
 #include <qx11info_x11.h>
@@ -68,6 +75,8 @@
 #include <ucontext.h>
 #endif
 
+#include "coreconfig_p.h"
+
 // Copy from klauncher_cmds
 typedef struct {
     long cmd;
@@ -94,20 +103,11 @@ static char *s_drkonqiPath = 0;
 static char *s_kdeinit_socket_file = 0;
 static KCrash::CrashFlags s_flags = 0;
 static int s_launchDrKonqi = -1; // -1=initial value 0=disabled 1=enabled
+Q_GLOBAL_STATIC(KCrash::CoreConfig, s_coreConfig)
 
 static void kcrashInitialize()
 {
-    if (s_launchDrKonqi == 0) { // disabled by the program itself
-        return;
-    }
-    const QStringList args = QCoreApplication::arguments();
-    if (qgetenv("KDE_DEBUG").isEmpty()
-        && qgetenv("KCRASH_AUTO_RESTARTED").isEmpty()) {
-        // enable drkonqi
-        KCrash::setDrKonqiEnabled(true);
-    }
-
-    KCrash::setApplicationFilePath(QCoreApplication::applicationFilePath());
+    KCrash::initialize();
 }
 Q_COREAPP_STARTUP_FUNCTION(kcrashInitialize)
 
@@ -121,6 +121,23 @@ LONG WINAPI win32UnhandledExceptionFilter(_EXCEPTION_POINTERS *exceptionInfo);
 #endif
 }
 
+void KCrash::initialize()
+{
+    if (s_launchDrKonqi == 0) { // disabled by the program itself
+        return;
+    }
+    const QStringList args = QCoreApplication::arguments();
+    if (!qEnvironmentVariableIsSet("KDE_DEBUG")
+        && !qEnvironmentVariableIsSet("KCRASH_AUTO_RESTARTED")) {
+        // enable drkonqi
+        KCrash::setDrKonqiEnabled(true);
+    }
+
+    KCrash::setApplicationFilePath(QCoreApplication::applicationFilePath());
+
+    s_coreConfig(); // Initialize.
+}
+
 void
 KCrash::setEmergencySaveFunction(HandlerType saveFunction)
 {
@@ -155,7 +172,7 @@ public:
         startTimer(10000); // 10 s
     }
 protected:
-    void timerEvent(QTimerEvent *event)
+    void timerEvent(QTimerEvent *event) Q_DECL_OVERRIDE
     {
         if (!s_crashHandler) { // not set meanwhile
             KCrash::setCrashHandler(KCrash::defaultCrashHandler);
@@ -172,7 +189,7 @@ KCrash::setFlags(KCrash::CrashFlags flags)
     if (s_flags & AutoRestart) {
         // We need at least the default crash handler for autorestart to work.
         if (!s_crashHandler) {
-            if (!qgetenv("KCRASH_AUTO_RESTARTED").isEmpty()) {
+            if (qEnvironmentVariableIsSet("KCRASH_AUTO_RESTARTED")) {
                 new KCrashDelaySetHandler;
             } else {
                 setCrashHandler(defaultCrashHandler);
@@ -207,16 +224,22 @@ void KCrash::setApplicationFilePath(const QString &filePath)
 
 void KCrash::setDrKonqiEnabled(bool enabled)
 {
-    s_launchDrKonqi = enabled ? 1 : 0;
+    const int launchDrKonqi = enabled ? 1 : 0;
+    if (s_launchDrKonqi == launchDrKonqi) {
+        return;
+    }
+    s_launchDrKonqi = launchDrKonqi;
     if (s_launchDrKonqi && !s_drkonqiPath) {
-        const QList<QString> paths = QFile::decodeName(qgetenv("LIBEXEC_PATH")).split(QLatin1Char(':'));
-        const QString exec = QStandardPaths::findExecutable(QLatin1String("drkonqi"), paths);
+        // search paths
+        const QStringList paths = QStringList()
+            << QFile::decodeName(qgetenv("LIBEXEC_PATH")).split(QLatin1Char(':'), QString::SkipEmptyParts) // env var is used first
+            << QCoreApplication::applicationDirPath() // then look where our application binary is located
+            << QLibraryInfo::location(QLibraryInfo::LibraryExecutablesPath) // look where libexec path is (can be set in qt.conf)
+            << QFile::decodeName(CMAKE_INSTALL_FULL_LIBEXECDIR); // look at our installation location
+        const QString exec = QStandardPaths::findExecutable(QStringLiteral("drkonqi"), paths);
         if (exec.isEmpty()) {
-            s_drkonqiPath = qstrdup(CMAKE_INSTALL_FULL_LIBEXECDIR "/drkonqi");
-            if (!QFile::exists(QLatin1String(s_drkonqiPath))) {
-                qWarning() << "Could not find drkonqi at" << s_drkonqiPath;
-                s_launchDrKonqi = 0;
-            }
+            qCDebug(LOG_KCRASH) << "Could not find drkonqi in search paths:" << paths;
+            s_launchDrKonqi = 0;
         } else {
             s_drkonqiPath = qstrdup(qPrintable(exec));
         }
@@ -277,7 +300,7 @@ KCrash::setCrashHandler(HandlerType handler)
     if (!s_kdeinit_socket_file) {
         // Prepare this now to avoid mallocs in the crash handler.
         char *display = getDisplay();
-        const QString socketFileName = QString::fromLatin1("kdeinit5_%1").arg(QLatin1String(display));
+        const QString socketFileName = QStringLiteral("kdeinit5_%1").arg(QLatin1String(display));
         free(display);
         QByteArray socketName = QFile::encodeName(QStandardPaths::writableLocation(QStandardPaths::RuntimeLocation) +
                                 QLatin1Char('/') + socketFileName);
@@ -509,6 +532,13 @@ KCrash::defaultCrashHandler(int sig)
         fprintf(stderr, "Unable to start Dr. Konqi\n");
     }
 
+    if (s_coreConfig->isProcess()) {
+        fprintf(stderr, "Re-raising signal for core dump handling.\n");
+        KCrash::setCrashHandler(0);
+        raise(sig);
+        // not getting here
+    }
+
     _exit(255);
 }
 
@@ -628,7 +658,10 @@ static bool startProcessInternal(int argc, const char *argv[], bool waitAndExit,
                 sleep(1);
             }
         }
-        _exit(253);
+        if (!s_coreConfig->isProcess()) {
+            // Only exit if we don't forward to core dumps
+            _exit(253);
+        }
     }
 
     return (pid > 0); //return true on success
@@ -643,7 +676,8 @@ static pid_t startFromKdeinit(int argc, const char *argv[])
     kcrash_launcher_header header;
     header.cmd = LAUNCHER_EXEC_NEW;
     const int BUFSIZE = 8192; // make sure this is big enough
-    char buffer[ BUFSIZE + 10 + 24 /*the env var*/ ];
+    const int CWDSIZE = 2000;
+    char buffer[ BUFSIZE + 10 + 24 /*the env var*/ + CWDSIZE+1 ];
     int pos = 0;
     long argcl = argc;
     memcpy(buffer + pos, &argcl, sizeof(argcl));
@@ -671,9 +705,18 @@ static pid_t startFromKdeinit(int argc, const char *argv[])
     long avoid_loops = 0;
     memcpy(buffer + pos, &avoid_loops, sizeof(avoid_loops));
     pos += sizeof(avoid_loops);
+
+    char cwd[CWDSIZE];
+    if (getcwd(cwd, CWDSIZE-1)) {
+        len = strlen(cwd) + 1; // include terminating \0
+        memcpy(buffer + pos, cwd, len);
+        pos += len;
+    }
+
     header.arg_length = pos;
     write_socket(socket, (char *) &header, sizeof(header));
     write_socket(socket, buffer, pos);
+
     if (read_socket(socket, (char *) &header, sizeof(header)) < 0
             || header.cmd != LAUNCHER_OK) {
         return 0;
@@ -752,6 +795,13 @@ static int read_socket(int sock, char *buffer, int len)
 static int openSocket()
 {
     struct sockaddr_un server;
+    if (!s_kdeinit_socket_file ||
+        strlen(s_kdeinit_socket_file) > (sizeof(server.sun_path) - 1))
+    {
+        fprintf(stderr, "kcrash: Unable to communicate with kdeinit5, socket name is %s!",
+                (s_kdeinit_socket_file) ? "too long" : "undefined");
+        return -1;
+    }
 
     /*
      * create the socket stream
